import pygame

def create_grid(rows, cols):
    """
    This function creates and returns a list of lists, intended to
    represent a grid. All of the grid cells are filled with False.
    """
    grid = []
    for r in range(0,rows):
        row = []
        for c in range(0, cols):
            row = row + [False]
        grid = grid + [row]
    return grid

def draw_grid(win, grid, size, margin):
    """
    This function takes a grid representation and draws it. False is
    represented as magenta, True as orange.
    """
    
    for row_index in range(0,len(grid)):
        for col_index in range(0, len(grid[row_index])):
            if grid[row_index][col_index]:
                color = pygame.color.Color("orange")
            else:
                color = pygame.color.Color("darkmagenta")
            x = col_index*(size+margin)
            y = row_index*(size+margin)
            pygame.draw.rect(win,color,(x, y, size, size))


def life_neighbors(board,rows,cols, r, c):
    """
    This function counts how many life neighbors a given cell in a
    Game of Life grid has.

    Parameters: the grid representation, the row number and column
    number we are interested in, the overall number of rows and
    columns.
    """
    count = 0
    for i in range(c-1,c+2):
        for j in range(r-1,r+2):
            if board[j%rows][i%cols] and (i != c or j != r):
                count += 1
    return count


def life_neighbors_alt(board,rows,cols, r, c):
    """
    An alternative implementation of life_neighbors.
    """
    count = 0

    if r==0:
        r_up = rows-1
    else:
        r_up = r-1

    if r==rows-1:
        r_down = 0
    else:
        r_down = r+1

    if c==0:
        c_left = cols-1
    else:
        c_left = c-1

    if c==cols-1:
        c_right = 0
    else:
        c_right = c+1
        
    if board[r_up][c_left]:
        count += 1
    if board[r_up][c]:
        count += 1
    if board[r_up][c_right]:
        count += 1
    if board[r][c_left]:
        count += 1
    if board[r][c_right]:
        count += 1
    if board[r_down][c_left]:
        count += 1
    if board[r_down][c]:
        count += 1
    if board[r_down][c_right]:
        count += 1

    return count


def next_generation(grid, rows, cols):
    """
    This function applies the Game of Life rules to a given grid. It
    returns the next generation of the grid.
    """
    new_board = create_grid(rows, cols)

    for r in range(0,rows):
        for c in range(0,cols):
            
            n = life_neighbors(grid,rows, cols, r, c)
            
            if grid[r][c] and (n < 2 or n > 3):
                new_board[r][c] = False
            elif not grid[r][c] and n == 3:
                new_board[r][c] = True
            else:
                new_board[r][c] = grid[r][c]

    return new_board

            
def run_game():
    
    ## Initialize the pygame submodules and set up the display window.

    pygame.init()

    width = 650
    height = 600
    my_win = pygame.display.set_mode((width,height))


    ## Initialize game objects.
    # the grid
    size = 10
    margin = 2
    rows = 50
    cols = 50
    grid = create_grid(rows, cols)

    # reset button
    reset_radius = 20
    reset_x = width - reset_radius - 2
    reset_y = reset_radius + 2

    # the go button
    go_radius = 20
    go_x = width - go_radius - 4
    go_y = 2 + 2* reset_radius + go_radius + 4
    go = False

    
    ## GAME LOOP starts
    keepGoing = True    
    while (keepGoing):

        ## Handle events.

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                keepGoing = False

            # handle mouse clicks
            elif event.type == pygame.MOUSEBUTTONDOWN:

                (mouse_x, mouse_y) = event.pos

                # within grid area
                if (mouse_x <= (margin + size) * rows
                    and mouse_x <= (margin + size) * cols):

                    col = mouse_x / (size + margin)
                    row = mouse_y / (size + margin)

                    grid[row][col] = not grid[row][col]

                # on reset button
                elif (mouse_x > reset_x - reset_radius and mouse_x < reset_x + reset_radius
                    and mouse_y > reset_y - reset_radius and mouse_y < reset_y + reset_radius):

                    grid = create_grid(rows, cols)

                # on start/stop button
                elif (mouse_x > go_x - go_radius and mouse_x < go_x + go_radius
                    and mouse_y > go_y - go_radius and mouse_y < go_y + go_radius):

                    go = not go

                    print go



        ## Update game objects.

        if go:
            grid = next_generation(grid, rows, cols)

        ## Draw picture and update display.

        my_win.fill(pygame.color.Color("lightblue"))
        
        draw_grid(my_win, grid, size, margin)
        pygame.draw.circle(my_win, pygame.color.Color("white"), (reset_x, reset_y), reset_radius)
        pygame.draw.circle(my_win, pygame.color.Color("red"), (go_x, go_y), go_radius)
        pygame.display.update()

    ## GAME LOOP ends

    pygame.quit()


## To test:
run_game()

# and try some configurations for which you know how they evolve
